@@ -1,9 +1,9 @@
-/* $Id: add_op.hpp 2625 2012-12-23 14:34:12Z bradbell $ */
+/* $Id: add_op.hpp 3321 2014-09-12 09:50:39Z bradbell $ */
 # ifndef CPPAD_ADD_OP_INCLUDED
 # define CPPAD_ADD_OP_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -13,10 +13,8 @@ A copy of this license is included in the COPYING file of this distribution.
 Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
-CPPAD_BEGIN_NAMESPACE
+namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup add_op_hpp add_op.hpp
-\{
 \file add_op.hpp
 Forward and reverse mode calculations for z = x + y.
 */
@@ -38,26 +36,67 @@ and the argument \a parameter is not used.
 
 template <class Base>
 inline void forward_addvv_op(
-	size_t        d           , 
+	size_t        p           , 
+	size_t        q           , 
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	Base*         taylor      )
 {
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddvvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddvvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( d < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+	CPPAD_ASSERT_UNKNOWN( p <= q  );
 
 	// Taylor coefficients corresponding to arguments and result
-	Base* x = taylor + arg[0] * nc_taylor;
-	Base* y = taylor + arg[1] * nc_taylor;
-	Base* z = taylor + i_z    * nc_taylor;
+	Base* x = taylor + arg[0] * cap_order;
+	Base* y = taylor + arg[1] * cap_order;
+	Base* z = taylor + i_z    * cap_order;
 
-	z[d] = x[d] + y[d];
+	for(size_t j = p; j <= q; j++)
+		z[j] = x[j] + y[j];
+}
+/*!
+Multiple directions forward mode Taylor coefficients for op = AddvvOp.
+
+The C++ source code corresponding to this operation is
+\verbatim
+	z = x + y
+\endverbatim
+In the documentation below,
+this operations is for the case where both x and y are variables
+and the argument \a parameter is not used.
+
+\copydetails forward_binary_op_dir
+*/
+
+template <class Base>
+inline void forward_addvv_op_dir(
+	size_t        q           , 
+	size_t        r           , 
+	size_t        i_z         ,
+	const addr_t* arg         ,
+	const Base*   parameter   ,
+	size_t        cap_order   ,
+	Base*         taylor      )
+{
+	// check assumptions
+	CPPAD_ASSERT_UNKNOWN( NumArg(AddvvOp) == 2 );
+	CPPAD_ASSERT_UNKNOWN( NumRes(AddvvOp) == 1 );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+	CPPAD_ASSERT_UNKNOWN( 0 < q  );
+
+	// Taylor coefficients corresponding to arguments and result
+	size_t num_taylor_per_var = (cap_order-1) * r + 1;
+	Base* x = taylor + arg[0] * num_taylor_per_var;
+	Base* y = taylor + arg[1] * num_taylor_per_var;
+	Base* z = taylor +    i_z * num_taylor_per_var;
+
+	size_t m = (q-1)*r + 1 ;
+	for(size_t ell = 0; ell < r; ell++)
+		z[m+ell] = x[m+ell] + y[m+ell];
 }
 
 /*!
@@ -79,19 +118,17 @@ inline void forward_addvv_op_0(
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	Base*         taylor      )
 {
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddvvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddvvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
 
 	// Taylor coefficients corresponding to arguments and result
-	Base* x = taylor + arg[0] * nc_taylor;
-	Base* y = taylor + arg[1] * nc_taylor;
-	Base* z = taylor + i_z    * nc_taylor;
+	Base* x = taylor + arg[0] * cap_order;
+	Base* y = taylor + arg[1] * cap_order;
+	Base* z = taylor + i_z    * cap_order;
 
 	z[0] = x[0] + y[0];
 }
@@ -116,7 +153,7 @@ inline void reverse_addvv_op(
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	const Base*   taylor      ,
 	size_t        nc_partial  ,
 	Base*         partial     )
@@ -124,9 +161,7 @@ inline void reverse_addvv_op(
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddvvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddvvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[0]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( d < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( d < cap_order );
 	CPPAD_ASSERT_UNKNOWN( d < nc_partial );
 
 	// Partial derivatives corresponding to arguments and result
@@ -156,36 +191,71 @@ this operations is for the case where x is a parameter and y is a variable.
 
 \copydetails forward_binary_op
 */
-
 template <class Base>
 inline void forward_addpv_op(
-	size_t        d           , 
+	size_t        p           , 
+	size_t        q           , 
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	Base*         taylor      )
 {
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddpvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddpvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( d < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+	CPPAD_ASSERT_UNKNOWN( p <= q );
 
 	// Taylor coefficients corresponding to arguments and result
-	Base* y = taylor + arg[1] * nc_taylor;
-	Base* z = taylor + i_z    * nc_taylor;
+	Base* y = taylor + arg[1] * cap_order;
+	Base* z = taylor + i_z    * cap_order;
 
-# if CPPAD_USE_FORWARD0SWEEP
-	CPPAD_ASSERT_UNKNOWN( d > 0 );
-	z[d] = y[d];
-# else
-	// Paraemter value
-	Base x = parameter[ arg[0] ];
-	if( d == 0 )
-		z[d] = x + y[d];
-	else	z[d] = y[d];
-# endif
+	if( p == 0 )
+	{	// Paraemter value
+		Base x = parameter[ arg[0] ];
+		z[0] = x + y[0];
+		p++;
+	}
+	for(size_t j = p; j <= q; j++)
+		z[j] = y[j];
+}
+/*!
+Multiple directions forward mode Taylor coefficients for op = AddpvOp.
+
+The C++ source code corresponding to this operation is
+\verbatim
+	z = x + y
+\endverbatim
+In the documentation below,
+this operations is for the case where x is a parameter and y is a variable.
+
+\copydetails forward_binary_op_dir
+*/
+template <class Base>
+inline void forward_addpv_op_dir(
+	size_t        q           , 
+	size_t        r           , 
+	size_t        i_z         ,
+	const addr_t* arg         ,
+	const Base*   parameter   ,
+	size_t        cap_order   ,
+	Base*         taylor      )
+{
+	// check assumptions
+	CPPAD_ASSERT_UNKNOWN( NumArg(AddpvOp) == 2 );
+	CPPAD_ASSERT_UNKNOWN( NumRes(AddpvOp) == 1 );
+	CPPAD_ASSERT_UNKNOWN( 0 < q );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+
+	// Taylor coefficients corresponding to arguments and result
+	size_t num_taylor_per_var = (cap_order-1) * r + 1;
+	size_t m                  = (q-1) * r + 1;
+	Base* y = taylor + arg[1] * num_taylor_per_var + m;
+	Base* z = taylor + i_z    * num_taylor_per_var + m;
+
+	for(size_t ell = 0; ell < r; ell++)
+		z[ell] = y[ell];
 }
 /*!
 Compute zero order forward mode Taylor coefficient for result of op = AddpvOp.
@@ -205,20 +275,19 @@ inline void forward_addpv_op_0(
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	Base*         taylor      )
 {
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddpvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddpvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
 
 	// Paraemter value
 	Base x = parameter[ arg[0] ];
 
 	// Taylor coefficients corresponding to arguments and result
-	Base* y = taylor + arg[1] * nc_taylor;
-	Base* z = taylor + i_z    * nc_taylor;
+	Base* y = taylor + arg[1] * cap_order;
+	Base* z = taylor + i_z    * cap_order;
 
 	z[0] = x + y[0];
 }
@@ -242,7 +311,7 @@ inline void reverse_addpv_op(
 	size_t        i_z         ,
 	const addr_t* arg         ,
 	const Base*   parameter   ,
-	size_t        nc_taylor   ,
+	size_t        cap_order   ,
 	const Base*   taylor      ,
 	size_t        nc_partial  ,
 	Base*         partial     )
@@ -250,8 +319,7 @@ inline void reverse_addpv_op(
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(AddvvOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(AddvvOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( d < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( d < cap_order );
 	CPPAD_ASSERT_UNKNOWN( d < nc_partial );
 
 	// Partial derivatives corresponding to arguments and result
@@ -267,6 +335,5 @@ inline void reverse_addpv_op(
 }
 
 
-/*! \} */
-CPPAD_END_NAMESPACE
+} // END_CPPAD_NAMESPACE
 # endif
