@@ -1,9 +1,9 @@
-/* $Id: discrete_op.hpp 2625 2012-12-23 14:34:12Z bradbell $ */
+/* $Id: discrete_op.hpp 3321 2014-09-12 09:50:39Z bradbell $ */
 # ifndef CPPAD_DISCRETE_OP_INCLUDED
 # define CPPAD_DISCRETE_OP_INCLUDED
 
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -14,17 +14,15 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 -------------------------------------------------------------------------- */
 
 
-CPPAD_BEGIN_NAMESPACE
+namespace CppAD { // BEGIN_CPPAD_NAMESPACE
 /*!
-\defgroup discrete_op_hpp discrete_op.hpp
-\{
 \file discrete_op.hpp
-Zero order forward mode for z = f(x) where f is piecewise constant.
+Forward mode for z = f(x) where f is piecewise constant.
 */
 
 
 /*!
-Compute zero order forward mode Taylor coefficient for result of op = DisOp.
+forward mode Taylor coefficient for result of op = DisOp.
 
 The C++ source code corresponding to this operation is
 \verbatim
@@ -38,6 +36,16 @@ base type for the operator; i.e., this operation was recorded
 using AD< \a Base > and computations by this routine are done using type 
 \a Base .
 
+\param p
+is the lowest order Taylor coefficient that will be calculated.
+
+\param q
+is the highest order Taylor coefficient that will be calculated.
+
+\param r
+is the number of directions, for each order, 
+that will be calculated (except for order zero wich only has one direction).
+
 \param i_z
 variable index corresponding to the result for this operation; 
 i.e. the row index in \a taylor corresponding to z. 
@@ -53,43 +61,62 @@ for this discrete function.
 variable index corresponding to the argument for this operator;
 i.e. the row index in \a taylor corresponding to x.
 
-\param nc_taylor
-number of colums in the matrix containing all the Taylor coefficients.
+\param cap_order
+maximum number of orders that will fit in the taylor array.
+
+\par tpv
+We use the notation
+<code>tpv = (cap_order-1) * r + 1</code>
+which is the number of Taylor coefficients per variable
 
 \param taylor
-\b Input: \a taylor [ \a arg[1] * \a nc_taylor + 0 ] 
-is the zero order Taylor coefficient corresponding to x. 
+\b Input: <code>taylor [ arg[1] * tpv + 0 ]</code>
+is the zero order Taylor coefficient corresponding to x.
 \n
-\b Output: \a taylor [ \a i_z * \a nc_taylor + 0 ] 
+\b Output: if <code>p == 0</code>
+<code>taylor [ i_z * tpv + 0 ]</code>
 is the zero order Taylor coefficient corresponding to z. 
+For k = max(p, 1), ... , q,
+<code>taylor [ i_z * tpv + (k-1)*r + 1 + ell ]</code>
+is the k-th order Taylor coefficient corresponding to z
+(which is zero).
 
 \par Checked Assertions where op is the unary operator with one result:
 \li NumArg(op) == 2
 \li NumRes(op) == 1
-\li \a arg[1] < \a i_z 
-\li \a 0 < \a nc_taylor
+\li q < cap_order
+\li 0 < r
 */
 template <class Base>
-inline void forward_dis_op_0(
+inline void forward_dis_op(
+	size_t        p           ,
+	size_t        q           ,
+	size_t        r           ,
 	size_t        i_z         ,
 	const addr_t* arg         ,
-	size_t        nc_taylor   , 
+	size_t        cap_order   , 
 	Base*         taylor      )
 {	
 	// check assumptions
 	CPPAD_ASSERT_UNKNOWN( NumArg(DisOp) == 2 );
 	CPPAD_ASSERT_UNKNOWN( NumRes(DisOp) == 1 );
-	CPPAD_ASSERT_UNKNOWN( size_t(arg[1]) < i_z );
-	CPPAD_ASSERT_UNKNOWN( 0 < nc_taylor );
+	CPPAD_ASSERT_UNKNOWN( q < cap_order );
+	CPPAD_ASSERT_UNKNOWN( 0 < r );
 
 	// Taylor coefficients corresponding to argument and result
-	Base* x = taylor + arg[1] * nc_taylor;
-	Base* z = taylor + i_z * nc_taylor;
+	size_t num_taylor_per_var = (cap_order-1) * r + 1;
+	Base* x = taylor + arg[1] * num_taylor_per_var;
+	Base* z = taylor +    i_z * num_taylor_per_var;
 
-	z[0]  = discrete<Base>::eval(arg[0], x[0]);
+	if( p == 0 )
+	{	z[0]  = discrete<Base>::eval(arg[0], x[0]);
+		p++;
+	}
+	for(size_t ell = 0; ell < r; ell++)
+		for(size_t k = p; k <= q; k++)
+			z[ (k-1) * r + 1 + ell ] = Base(0);
 }
 
 
-/*! \} */
-CPPAD_END_NAMESPACE
+} // END_CPPAD_NAMESPACE
 # endif
