@@ -1,6 +1,6 @@
-/* $Id: mat_mul.cpp 2506 2012-10-24 19:36:49Z bradbell $ */
+/* $Id: mat_mul.cpp 3311 2014-05-28 16:21:08Z bradbell $ */
 /* --------------------------------------------------------------------------
-CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-12 Bradley M. Bell
+CppAD: C++ Algorithmic Differentiation: Copyright (C) 2003-14 Bradley M. Bell
 
 CppAD is distributed under multiple licenses. This distribution is under
 the terms of the 
@@ -13,14 +13,13 @@ Please visit http://www.coin-or.org/CppAD/ for information on other licenses.
 $begin cppad_mat_mul.cpp$$
 $spell
 	resize
-	info info
 	nr
 	nc
 	cppad
 	mul
 	hpp
 	bool
-	retape
+	onetape
 	sq
 	var
 	std::cout
@@ -48,8 +47,11 @@ $codep */
 # include <cppad/cppad.hpp>
 # include <cppad/speed/mat_sum_sq.hpp>
 # include <cppad/speed/uniform_01.hpp>
-# include "../../example/mat_mul.hpp"
-# include "print_optimize.hpp"
+# include <cppad/example/matrix_mul.hpp>
+
+// Note that CppAD uses global_memory at the main program level
+extern bool
+	global_onetape, global_atomic, global_optimize;
 
 bool link_mat_mul(
 	size_t                           size     , 
@@ -59,9 +61,6 @@ bool link_mat_mul(
 	CppAD::vector<double>&           dz
 )
 {
-	// speed test global option values
-	extern bool global_retape, global_atomic, global_optimize;
-
 	// -----------------------------------------------------
 	// setup
 	typedef CppAD::AD<double>           ADScalar; 
@@ -80,23 +79,14 @@ bool link_mat_mul(
 	w[0] = 1.;
 
 	// user atomic information
-	size_t   info_id = info_.size();  
 	CppAD::vector<ADScalar> ax(2 * n), ay(n);
-	call_info info;
-	if( global_atomic )
-	{	info.nr_result = size;
-		info.n_middle  = size;
-		info.nc_result = size;
-		info_.push_back(info);
-	}
-	
-	// use the unspecified fact that size is non-decreasing between calls
-	static size_t previous_size = 0;
-	bool print    = (repeat > 1) & (previous_size != size);
-	previous_size = size;
+	size_t nr_result = size;
+	size_t n_middle  = size;
+	size_t nc_result = size;
+	matrix_mul atom_mul(nr_result, n_middle, nc_result);
 
 	// ------------------------------------------------------
-	if( global_retape ) while(repeat--)
+	if( ! global_onetape ) while(repeat--)
 	{	// get the next matrix
 		CppAD::uniform_01(n, x);
 		for( j = 0; j < n; j++)
@@ -114,7 +104,7 @@ bool link_mat_mul(
 				ax[j+n] = X[j];
 			}
 			// Y = X * X
-			mat_mul(info_id, ax, ay);
+			atom_mul(ax, ay);
 			Z[0] = 0.;
 			for(j = 0; j < n; j++)
 				Z[0] += ay[j];
@@ -123,9 +113,7 @@ bool link_mat_mul(
 		f.Dependent(X, Z);
 
 		if( global_optimize )
-		{	print_optimize(f, print, "cppad_mat_mul_optimize", size);
-			print = false;
-		}
+			f.optimize();
 
 		// evaluate and return gradient using reverse mode
 		z  = f.Forward(0, x);
@@ -149,7 +137,7 @@ bool link_mat_mul(
 				ax[j+n] = X[j];
 			}
 			// Y = X * X
-			mat_mul(info_id, ax, ay);
+			atom_mul(ax, ay);
 			Z[0] = 0.;
 			for(j = 0; j < n; j++)
 				Z[0] += ay[j];
@@ -159,9 +147,7 @@ bool link_mat_mul(
 		f.Dependent(X, Z);
 
 		if( global_optimize )
-		{	print_optimize(f, print, "cppad_mat_mul_optimize", size);
-			print = false;
-		}
+			f.optimize();
 		while(repeat--)
 		{	// get a next matrix
 			CppAD::uniform_01(n, x);
@@ -175,7 +161,6 @@ bool link_mat_mul(
 	// Free temporary work space. (If there are future calls to 
 	// mat_mul they would create new temporary work space.)
 	CppAD::user_atomic<double>::clear();
-	info_.clear();
 
 	return true;
 }
